/*
 *    Lightscheme "My Long Pulse"
 *    with battery check
 *
 *    Hardware:
 *        Board: Arudino MINI (or UNO or Leonardo or ...)
 *        PWM-Pin: D9  ---  R(150E) ---  Transistor BC337
 *        "Battery sensor":  Battery(+)  ---  R(10k) -----  A0  -----  R(2k2) ---  Battery(-)
 *
 *    Author: Udo Somma
 *    This code in the public domain.
 *    So feel free to use it, change it, redistiribute it...
 *
 *    Last update: 2013-11-15
 */




#include <avr/sleep.h>        //  include the sleep library
                              //  so the board can turn off if battery is low

int PulsLED    = 9   ;        //  the pulse-light-pin
int brightness = 30  ;        //  how bright the LED is
int fadeAmount = 1   ;        //  how many points to fade the LED by
int fadeMax    = 255 ;        //  how far shall we go: upper end
int fadeMin    = 15  ;        //  how far shall we go: lower end

int analogPin = 0;            //  the pin reading the battery voltage (A0)
int batVal;                   //  the battery voltage value
int lowBatVal = 680;          //  value at "battery sensor" if less than 4 volts 

 
 
void setup()
{ 
   pinMode(PulsLED, OUTPUT);
   pinMode(13, OUTPUT);            //  don't waste energy  (with other boards 
   digitalWrite(13, LOW);          //  than the MINI. the MINI has no LED on D13). 
   analogReference(INTERNAL);      //  use the built-in reference: equal to 1.1 volts
}                                  



void loop()
{  
   analogWrite(PulsLED, brightness);              //  set the brightness of LEDs
   brightness = brightness + fadeAmount;          //  change the brightness for next time through the loop
   
   if (brightness == fadeMax)  {                  //  if we are at full brighness...
       delay(5500);                               //  ...let us stay some time at full brightness. 

       batVal = analogRead(analogPin);            //  read the input pin (battery voltage)
       if (batVal < lowBatVal)  {                 //  if battery voltage is too low 
           analogWrite(PulsLED, 0);               //  set the brightness of LED to zero
           delay(500);
           set_sleep_mode(SLEEP_MODE_PWR_DOWN);   // sleep mode is set here
           sleep_enable();                        // enables the sleep bit in the mcucr register
        				          // so sleep is possible. just a safety pin 
           sleep_mode();                          // now go to sleep!!!
       }
   }
   
   if (brightness <= 60)  {                          //  next steps will help to smooth the fade.
       delay(7);                                     //  led brightness is not linear with pwm duty cycle.
   }                                                 //  the relative brighness changes faster with lower 
                                                     //  values, so we keep lower values some milliseconds
   if (brightness <= 40)  {                          //  longer. this is just a quick & dirty coding, but it 
       delay(7);                                     //  works fine for me. 
   }                                                 //  alternative (more elegant): use a lookup table.

   if (brightness <= 20)  {
       delay(12);
   }
   
   if (brightness == fadeMin  ||  brightness == fadeMax)  {    //  reverse the direction of the fading at 
       fadeAmount = -fadeAmount ;                              //  the both ends of the fade. 
   } 
       
   delay(7) ;
}


